/***********************************************************************/
/*                                                                     */
/*                           Objective Caml                            */
/*                                                                     */
/*            Xavier Leroy, projet Cristal, INRIA Rocquencourt         */
/*                                                                     */
/*  Copyright 1996 Institut National de Recherche en Informatique et   */
/*  en Automatique.  All rights reserved.  This file is distributed    */
/*  under the terms of the GNU Library General Public License, with    */
/*  the special exception on linking described in file ../LICENSE.     */
/*                                                                     */
/***********************************************************************/

/* $Id: digest.c,v 1.19 2005/09/22 14:21:50 xleroy Exp $ */

/* Caml bindings for the SHA256 message digest implementation. */
/* Copyright (C) 2006-2007 Peter Selinger */

#include <mlvalues.h>
#include <alloc.h>
#include <memory.h>
#include <custom.h>

#include "../src/sha256.h"
#include "caml_sha256.h"

struct custom_operations ops = {
  /* identifier */  "www.mathstat.dal.ca/~selinger/sha256/0.1",
  /* finalize */    NULL,
  /* compare */     custom_compare_default,
  /* hash */        custom_hash_default,
  /* serialize */   custom_serialize_default,
  /* deserialize */ custom_deserialize_default,
};

/* return an initialized sha256 heap object */
CAMLprim value caml_sha256_new (void)
{
  CAMLparam0();
  CAMLlocal1(state);
  sha256_state_t *s;

  state = alloc_custom(&ops, sizeof(sha256_state_t), 0, 1);
  s = (sha256_state_t *) Data_custom_val(state);
  sha256_init(s);

  CAMLreturn(state);
}

/* hash len bytes, starting from offset ofs in string str */
CAMLprim value caml_sha256_data (value state, value str, value ofs, value len)
{
  CAMLparam4(state, str, ofs, len);
  sha256_state_t *s = (sha256_state_t *) Data_custom_val(state);

  sha256_bytes(s, &Byte_u(str, Long_val(ofs)), Long_val(len));
  
  CAMLreturn(Val_unit);
}

/* finalize hash and return hash as a string. */
CAMLprim value caml_sha256_finish (value state) {
  CAMLparam1(state);
  CAMLlocal1(res);
  sha256_state_t *s = (sha256_state_t *) Data_custom_val(state);

  sha256_finish(s);
  res = alloc_string(32);
  sha256_gethash(s, &Byte_u(res, 0));

  CAMLreturn(res);
}
