#!/bin/sh

# Copyright (C) 2006-2007 Peter Selinger.
# This file is part of the hashfunctions package. It is free software
# and it is covered by the GNU general public license. See the file
# COPYING for details.

# Run tests using the sample data from FIPS 180-2, see
# http://csrc.nist.gov/publications/fips/fips180-2/fips180-2withchangenotice.pdf

NAME="$0"
SHA256="./main"
FIPSTEST="../check/fipstest"
RV=0

hashtest() {
    N="$1"
    C="$2"
    H="$3"
    set `"$FIPSTEST" "$N" | "$C"`
    HASH="$1"
    if [ "$HASH" != "$H" ]; then
	echo "$NAME:$LINE: $C Test $N failed" >&2
	echo "returned $HASH" >&2
	echo "expected $H" >&2
	RV=1
    elif [ "$VERBOSE" != "" ]; then
	echo "$NAME:$LINE: $C Test $N passed" >&2
    fi
}

stringtest() {
    S="$1"
    C="$2"
    H="$3"
    set `"$C" -s "$S"`
    HASH="$1"
    if [ "$HASH" != "$H" ]; then
	echo "$NAME:$LINE: $C String test failed" >&2
	echo "returned $HASH" >&2
	echo "expected $H" >&2
	RV=1
    elif [ "$VERBOSE" != "" ]; then
	echo "$NAME:$LINE: $C String test passed" >&2
    fi
}

alias hashtest='LINE=$LINENO; hashtest'
alias stringtest='LINE=$LINENO; stringtest'

hashtest 1 "$SHA256" "ba7816bf8f01cfea414140de5dae2223b00361a396177a9cb410ff61f20015ad"
hashtest 2 "$SHA256" "248d6a61d20638b8e5c026930c3e6039a33ce45964ff2167f6ecedd419db06c1"
hashtest 3 "$SHA256" "cdc76e5c9914fb9281a1c7e284d73e67f1809a48a497200e046d39ccc7112cd0"

hashtest 4 "$SHA256" "68aa2e2ee5dff96e3355e6c7ee373e3d6a4e17f75f9518d843709c0c9bc3e3d4"
hashtest 5 "$SHA256" "175ee69b02ba9b58e2b0a5fd13819cea573f3940a94f825128cf4209beabb4e8"
hashtest 6 "$SHA256" "97dbca7df46d62c8a422c941dd7e835b8ad3361763f7e9b2d95f4f0da6e1ccbc"

stringtest "abc" "$SHA256" "ba7816bf8f01cfea414140de5dae2223b00361a396177a9cb410ff61f20015ad"
stringtest "abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq" "$SHA256" "248d6a61d20638b8e5c026930c3e6039a33ce45964ff2167f6ecedd419db06c1"

exit "$RV"
