#!/bin/sh

# Copyright (C) 2006-2007 Peter Selinger.
# This file is part of the hashfunctions package. It is free software
# and it is covered by the GNU general public license. See the file
# COPYING for details.

# Run tests similar to NIST's SHA Validation System and the FIPS
# tests, applied to the CRC32 checksumming algorithm.

if [ -z "$srcdir" ]; then
    srcdir=.
fi

. "$srcdir/missing.sh"

NAME="$0"
SHAVSTEST="./shavstest"
RV=0
TMPDIR="${TEMPDIR:-/tmp}"
TMP=`mktemp "$TMPDIR/shavstest.XXXXXX"`
DATA="$srcdir"/data

cleanup () {
    rm -f "$TMP"
}

trap cleanup EXIT
trap cleanup INT

shavstest() {
    A="$1"
    F="$2"
    cat "$DATA/$F" | "$SHAVSTEST" "$A" > "$TMP"
    diff "$DATA/$F" "$TMP" > /dev/null
    if [ "$?" -ne "0" ]; then
	echo "$F: failed" >&2
	RV=1
    elif [ "$VERBOSE" != "" ]; then
	echo "$F: passed" >&2
    fi
}

alias shavstest='LINE=$LINENO; shavstest'

# a "Short Msg" test is sufficient for CRC32, because the
# implementation is insensitive to overall message length.

shavstest crc32 CRC32ShortMsg.txt

CRC32="../src/crc32"
FIPSTEST="./fipstest"

hashtest() {
    N="$1"
    C="$2"
    H="$3"
    HASH=`"$FIPSTEST" "$N" | "$C"`
    if [ "$HASH" != "$H" ]; then
	echo "$NAME:$LINE: $C Test $N failed" >&2
	echo "returned $HASH" >&2
	echo "expected $H" >&2
	RV=1
    elif [ "$VERBOSE" != "" ]; then
	echo "$NAME:$LINE: $C Test $N passed" >&2
    fi
}

alias hashtest='LINE=$LINENO; hashtest'

hashtest 1 "$CRC32" "2c17398c"
hashtest 2 "$CRC32" "8fcb550e"
hashtest 3 "$CRC32" "ee6d4270"
hashtest 4 "$CRC32" "6ed82b7f"
hashtest 5 "$CRC32" "356936e3"
hashtest 6 "$CRC32" "71828bf2"
hashtest 7 "$CRC32" "44190b0d"
hashtest 8 "$CRC32" "073db21b"
hashtest 9 "$CRC32" "d56e1e23"

exit "$RV"
