#!/bin/sh

# Copyright (C) 2006-2007 Peter Selinger.
# This file is part of the hashfunctions package. It is free software
# and it is covered by the GNU general public license. See the file
# COPYING for details.

# Run the tests from NIST's SHA Validation System [2], using sample
# "SHA Test Vectors for Hashing Bit-Oriented Messages" [3] supplied by
# NIST. See:
# [1] http://csrc.nist.gov/cryptval/
# [2] http://csrc.nist.gov/cryptval/shs/SHAVS.pdf
# [3] http://csrc.nist.gov/cryptval/shs/shabittestvectors.zip

if [ -z "$srcdir" ]; then
    srcdir=.
fi

. "$srcdir/missing.sh"

NAME="$0"
SHAVSTEST="./shavstest"
RV=0
TMPDIR="${TEMPDIR:-/tmp}"
TMP=`mktemp "$TMPDIR/shavstest.XXXXXX"`
DATA="$srcdir"/data

cleanup () {
    rm -f "$TMP"
}

trap cleanup EXIT
trap cleanup INT

shavstest() {
    A="$1"
    F="$2"
    cat "$DATA/$F" | "$SHAVSTEST" "$A" > "$TMP"
    diff "$DATA/$F" "$TMP" > /dev/null
    if [ "$?" -ne "0" ]; then
	echo "$F: failed" >&2
	RV=1
    elif [ "$VERBOSE" != "" ]; then
	echo "$F: passed" >&2
    fi
}

alias shavstest='LINE=$LINENO; shavstest'

shavstest sha1 SHA1ShortMsg.txt
shavstest sha1 SHA1LongMsg.txt
shavstest sha1 SHA1Monte.txt
shavstest sha256 SHA256ShortMsg.txt
shavstest sha256 SHA256LongMsg.txt
shavstest sha256 SHA256Monte.txt

exit "$RV"
